namespace Org.BouncyCastle.Crypto.Modes
{
	/**
	* A block cipher mode that includes authenticated encryption with a streaming mode and optional associated data.
	* @see org.bouncycastle.crypto.params.AeadParameters
	*/
	public interface IAeadBlockCipher
	{
		/**
		* Initialise the underlying cipher. Parameter can either be an AeadParameters or a ParametersWithIV object.
		*
		* @param forEncryption true if we are setting up for encryption, false otherwise.
		* @param parameters the necessary parameters for the underlying cipher to be initialised.
		* @exception ArgumentException if the parameters argument is inappropriate.
		*/
		void Init(bool forEncryption, ICipherParameters parameters);

		/**
		* Return the name of the algorithm.
		* 
		* @return the algorithm name.
		*/
		string AlgorithmName { get; }

//		/**
//		* Return the cipher this object wraps.
//		*
//		* @return the cipher this object wraps.
//		*/
//		public IBlockCipher GetUnderlyingCipher();

		/**
		* Encrypt/decrypt a single byte.
		*
		* @param input the byte to be processed.
		* @param outBytes the output buffer the processed byte goes into.
		* @param outOff the offset into the output byte array the processed data starts at.
		* @return the number of bytes written to out.
		* @exception DataLengthException if the output buffer is too small.
		*/
		int ProcessByte(byte input, byte[] outBytes, int outOff);

		/**
		* Process a block of bytes from in putting the result into out.
		*
		* @param inBytes the input byte array.
		* @param inOff the offset into the in array where the data to be processed starts.
		* @param len the number of bytes to be processed.
		* @param outBytes the output buffer the processed bytes go into.
		* @param outOff the offset into the output byte array the processed data starts at.
		* @return the number of bytes written to out.
		* @exception DataLengthException if the output buffer is too small.
		*/
		int ProcessBytes(byte[] inBytes, int inOff, int len, byte[] outBytes, int outOff);

		/**
		* Finish the operation either appending or verifying the MAC at the end of the data.
		*
		* @param outBytes space for any resulting output data.
		* @param outOff offset into out to start copying the data at.
		* @return number of bytes written into out.
		* @throws InvalidOperationException if the cipher is in an inappropriate state.
		* @throws InvalidCipherTextException if the MAC fails to match.
		*/
		int DoFinal(byte[] outBytes, int outOff);

		/**
		* Return the value of the MAC associated with the last stream processed.
		*
		* @return MAC for plaintext data.
		*/
		byte[] GetMac();

		/**
		* Return the size of the output buffer required for a ProcessBytes
		* an input of len bytes.
		*
		* @param len the length of the input.
		* @return the space required to accommodate a call to ProcessBytes
		* with len bytes of input.
		*/
		int GetUpdateOutputSize(int len);

		/**
		* Return the size of the output buffer required for a ProcessBytes plus a
		* DoFinal with an input of len bytes.
		*
		* @param len the length of the input.
		* @return the space required to accommodate a call to ProcessBytes and DoFinal
		* with len bytes of input.
		*/
		int GetOutputSize(int len);

		/**
		 * Reset the cipher. After resetting the cipher is in the same state
		 * as it was after the last init (if there was one).
		 */
		void Reset();
	}
}
